@@ -1,11 +1,11 @@
 # Copyright 2021-2022 Boris Shminke
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #     https://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -18,7 +18,7 @@ Saturation Environment
 import dataclasses
 import os
 import random
-from typing import Dict, List, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple
 
 import numpy as np
 import orjson
@@ -41,6 +41,7 @@ from gym_saturation.parsing.tptp_parser import TPTPParser
 
 STATE_DIFF_UPDATED = "state_diff_updated"
 POSITIVE_ACTIONS = "positive_actions"
+PROBLEM_FILENAME = "problem_filename"
 MAX_CLAUSES = 100000
 
 
@@ -71,7 +72,7 @@ class SaturationEnv(Env):
     >>> print(env.render())
     cnf(this_is_a_test_case_1, hypothesis, this_is_a_test_case(test_constant)).
     cnf(this_is_a_test_case_2, hypothesis, ~this_is_a_test_case(test_constant)).
-    cnf(test_axiom, axiom, test_constant = X0).
+    cnf(test_axiom, axiom, test_constant = test_constant_2).
     cnf(test_axiom_2, axiom, ~test_constant = 0).
 
     ``ansi`` mode returns a JSON representation of the state
@@ -106,41 +107,37 @@ class SaturationEnv(Env):
 
     there is no reward until the end of an episode
 
-    >>> env.step(1)[1:3]
+    >>> (reward, done)
     (0.0, False)
 
     if a proof is found, then reward is ``+1``
 
-    >>> env.step(4)[1:3]
+    >>> env.step(1)[1:3]
     (1.0, True)
 
     TSTP proof is now available (one can add ``include`` directive before it
     for validation purposes)
 
     >>> print(TPTPParser().parse(env.tstp_proof, "")[0])  # doctest: +ELLIPSIS
-    cnf(..., lemma, $false(), inference(resolution, [], [this_is_a_test_case_1, this_is_a_test_case_2])).
+    cnf(..., lemma, $false, inference(resolution, [], [this_is_a_test_case_1, this_is_a_test_case_2])).
 
     the relevant actions are filtered too
 
-    >>> env.positibe_actions
+    >>> env.positive_actions
     (0, 1, 4)
 
     the total number of clauses in the state is limited by the ``max_clauses``
     parameter. Let's try setting it and repeating the same solution of the same
     problem:
 
-    >>> env = SaturationEnv(problem_list, max_clauses=4)
+    >>> env = SaturationEnv(problem_list, max_clauses=3)
     >>> _ = env.seed(0)
-    >>> _ = env.reset()
+    >>> old_obs = env.reset()
 
-    after the first step, the number of clauses in the state doesn't change
-
-    >>> old_obs, _, _, _ = env.step(0)
-
-    but during the second step we bypass ``max_clauses`` by one, so everything
+    after the first step we bypass ``max_clauses`` by one, so everything
     is rolled back:
 
-    >>> obs, reward, done, _ = env.step(1)
+    >>> obs, reward, done, _ = env.step(0)
 
     the episode is consedered to be finished
 
@@ -175,13 +172,15 @@ class SaturationEnv(Env):
                 "real_obs": ClauseSpace(),
             }
         )
+        self.problem: Optional[str] = None
+        self._tptp_parser = TPTPParser()
 
     def _init_clauses(self) -> Tuple[Clause, ...]:
-        problem = random.choice(self.problem_list)
-        tptp_folder = os.path.join(os.path.dirname(problem), "..", "..")
-        with open(problem, "r", encoding="utf-8") as problem_file:
+        self.problem = random.choice(self.problem_list)
+        tptp_folder = os.path.join(os.path.dirname(self.problem), "..", "..")
+        with open(self.problem, "r", encoding="utf-8") as problem_file:
             problem_text = problem_file.read()
-        parsed_clauses = TPTPParser().parse(problem_text, tptp_folder)
+        parsed_clauses = self._tptp_parser.parse(problem_text, tptp_folder)
         return tuple(
             dataclasses.replace(
                 clause,
@@ -262,44 +261,47 @@ class SaturationEnv(Env):
             + [(action, orjson.dumps(self._state[action]))]
         )
 
-    def step(self, action: int) -> Tuple[dict, float, bool, dict]:
+    def _proof_found_result(
+        self, reward: float, done: bool, info: Dict[str, Any]
+    ) -> Tuple[float, bool, Dict[str, Any]]:
+        if not done:
+            if tuple(True for clause in self._state if clause.literals == ()):
+                info[POSITIVE_ACTIONS] = self.positive_actions
+                return 1.0, True, info
+        return reward, done, info
+
+    def _max_clauses_result(
+        self,
+        old_state: Tuple[Clause, ...],
+        reward: float,
+        done: bool,
+        info: Dict[str, Any],
+    ) -> Tuple[float, bool, Dict[str, Any]]:
+        if not done:
+            if len(self._state) > self.action_space.n:
+                self._state = old_state
+                info.pop(STATE_DIFF_UPDATED)
+                return reward, True, info
+        return reward, done, info
+
+    def step(self, action: int) -> Tuple[dict, float, bool, Dict[str, Any]]:
         old_state = self._state
         if self._state[action].processed:
             raise ValueError(f"action {action} is not valid")
-        if self._state[action].literals == ():
-            self._state = (
-                self._state[:action]
-                + (dataclasses.replace(self._state[action], processed=True),)
-                + self._state[action + 1 :]
-            )
-            return (
-                self.state,
-                1.0,
-                True,
-                {
-                    STATE_DIFF_UPDATED: {
-                        action: orjson.dumps(self._state[action])
-                    },
-                    POSITIVE_ACTIONS: self.positibe_actions,
-                },
-            )
         updated = self._do_deductions(action)
-        if min(
+        reward = 0.0
+        info = {STATE_DIFF_UPDATED: updated, PROBLEM_FILENAME: self.problem}
+        done = min(
             [
                 False if clause.processed is None else clause.processed
                 for clause in self._state
             ]
-        ):
-            return self.state, 0.0, True, {STATE_DIFF_UPDATED: updated}
-        if len(self._state) > self.action_space.n:
-            self._state = old_state
-            return self.state, 0.0, True, {}
-        return (
-            self.state,
-            0.0,
-            False,
-            {STATE_DIFF_UPDATED: updated},
         )
+        reward, done, info = self._proof_found_result(reward, done, info)
+        reward, done, info = self._max_clauses_result(
+            old_state, reward, done, info
+        )
+        return self.state, reward, done, info
 
     @property
     def last_birth_step(self) -> int:
@@ -357,7 +359,7 @@ class SaturationEnv(Env):
         )
 
     @property
-    def positibe_actions(self) -> Tuple[int, ...]:
+    def positive_actions(self) -> Tuple[int, ...]:
         """
         :returns: a sequence of actions which contributed to the proof found
             (if found; raises an error otherwise)
